;;; hypersrc-graphics.el --- Functions for graphics file conversion

;;; $Id: hypersrc-graphics.el,v 0.5 1995/05/23 12:52:07 job Exp $

(defvar hypersrc-image-xpm
  "/* XPM */
static char * image_xpm[] = {
\"48 32 3 1\",
\" 	c #00000000FFFF\",
\".	c #FFFFFFFFFFFF\",
\"X	c #000000000000\",
\"                                                \",
\" .............................................. \",
\" .............................................. \",
\" .............................................. \",
\" .............................................. \",
\" .............................................. \",
\" .............................................. \",
\" .............................................. \",
\" .............................................. \",
\" .............................................. \",
\" .............................................. \",
\" ...XXX.XX.......XX....X......XXX.X..XXXXXXX... \",
\" ....X...XX.....XX.....X.....XX..XX...X....X... \",
\" ....X...XX.....XX....X.X....X....X...X........ \",
\" ....X...X.X...X.X....X.X...X.........X...X.... \",
\" ....X...X.X...X.X...X...X..X....XXX..XXXXX.... \",
\" ....X...X..X.X..X...XXXXX..X.....X...X...X.... \",
\" ....X...X..X.X..X...X...X...X....X...X........ \",
\" ....X...X...X...X..X.....X..XX..XX...X....X... \",
\" ...XXX.XXX..X..XXXXXX...XXX..XXXX...XXXXXXX... \",
\" .............................................. \",
\" .............................................. \",
\" .............................................. \",
\" .............................................. \",
\" .............................................. \",
\" .............................................. \",
\" .............................................. \",
\" .............................................. \",
\" .............................................. \",
\" .............................................. \",
\" .............................................. \",
\"                                                \"};"
  "String representing a default icon for external images")


(defun hypersrc-filename-suffix (filename)
  "Return suffix of FILENAME.
If FILENAME is \"/some/path/foo.bar\", return \"bar\"."
  (if (string-match "[^/]\\.\\([^.]+\\)$" filename)				; Match suffix (Note: "/foo/.bar" has no suffix!)
      (substring filename (match-beginning 1))					; Return suffix string if file has suffix
    ""))									; Otherwise return the empty string
  
(defun hypersrc-filename-no-suffix (filename)
  "Return FILENAME without suffix.
If FILENAME is \"/some/path/foo.bar\", return \"/some/path/foo\"."
  (if (string-match "[^/]\\.\\([^.]+\\)$" filename)				; Match suffix (Note: "/foo/.bar" has no suffix!)
      (substring filename 0 (1- (match-beginning 1)))				; Return filename without suffix if existent
    filename))									; Otherwise return complete string


(defun hypersrc-select-image-type-entry (filename)
  "Return entry of image-type-alist to use for image file FILENAME.
Entry is selected based on FILENAME's suffix.
If suffix is not found in the alist, nil is returned."
  (assoc (hypersrc-filename-suffix filename) hypersrc-image-type-alist))	; Get corresponding entry from alist of image types


(defun hypersrc-image-editor-command (filename)
  "Return name of editor program to use for image file FILENAME.
The name is selected from image-type-alist, based on FILENAME's suffix.
If suffix is not found in the alist, or the editor entry is nil,
nil is returned."
  (if (hypersrc-select-image-type-entry filename)
      (elt (hypersrc-select-image-type-entry filename) 1)))


(defun hypersrc-shell-command (command &optional flag)
  "Execute string COMMAND in inferior shell; display output, if any.
If COMMAND ends in ampersand, execute it asynchronously.
 
Optional second arg non-nil (prefix arg, if interactive)
means insert output in current buffer after point (leave mark after it).
This cannot be done asynchronously.

This is a copy of the original shell-command in simple.el.
The only change is that it pushes the mark only if called interactively.
(Actually this seems to have been overlooked in the original routine...)"
  (interactive (list (read-shell-command (gettext "Shell command: "))
		     current-prefix-arg))
  (if flag
      (progn (barf-if-buffer-read-only)

	     ;; HERE IS THE FIRST DIFFERENCE TO THE ORIGINAL SHELL-COMMAND:
             ;; Don't muck with mark:
	     ;; (push-mark)
	     
             ;; We do not use -f for csh; we will not support broken use of
	     ;; .cshrcs.  Even the BSD csh manual says to use
	     ;; "if ($?prompt) exit" before things which are not useful
	     ;; non-interactively.  Besides, if someone wants their other
	     ;; aliases for shell commands then they can still have them.
	     (call-process shell-file-name nil t nil
			   "-c" command)
	     ;; HERE IS THE SECOND DIFFERENCE TO THE ORIGINAL SHELL-COMMAND:
             ;; Don't muck with mark:
	     ;; (exchange-point-and-mark t)
    )

    ;; Preserve the match data in case called from a program.
    (let ((data (match-data)))
      (unwind-protect
	  (if (string-match "[ \t]*&[ \t]*$" command)
	      ;; Command ending with ampersand means asynchronous.
	      (progn
 		(require 'background) ; whizzy comint background code
 		(background (substring command 0 (match-beginning 0))))
	    (shell-command-on-region (point) (point) command nil))
	(store-match-data data)))))


(defun hypersrc-image-converter-command (filename)
  "Return name of converter program to use for image file FILENAME.
The name is selected from image-type-alist, based on FILENAME's suffix.
If suffix is not found in the alist, an error is signaled."
  (or 
   (elt (hypersrc-select-image-type-entry filename) 2)
   (error
    "HyperSource error: Please add converter name for \".%s\" files to 'hypersrc-image-type-alist."
    (hypersrc-filename-suffix filename))))

;;; This routine has been changed (see below) to avoid creating pixmaps
;;; from strings (older XPM libraries can't do this)
;;;
;(defun hypersrc-convert-any-to-pixmap (filename)
;  "Take an arbitrary graphics file FILENAME, convert it to XPM format if
;necessary, and return a pixmap object."
;  (let (converter shell-file-name anytoppm-commandline ppmtoxpm-commandline	; Local variables
;		  xpm-data)
;    (setq converter (hypersrc-image-converter-command filename))		; Find converter to use for filename
;    (if (not converter)								; If converter is nil, the file is XPM compatible,
;	(make-pixmap filename)							;   so just create the pixmap from it and return
;      (save-excursion								; ELSE:
;	(set-buffer (setq temp-buffer (get-buffer-create "*PPM/XPM*")))		; Create temp buffer to hold converted image data
;	(setq shell-file-name "/bin/sh")					; Use "sh" for stderr redirection
;	(setq anytoppm-commandline (concat converter " 2>/dev/null " filename))	; Construct shell command to convert to ppm
;	(shell-command anytoppm-commandline t)					; Convert image to ppm, write output in temp buffer
;	(setq ppmtoxpm-commandline						; Construct shell command to convert ppm to xpm
;	      (concat hypersrc-ppmtoxpm-command " 2>/dev/null"))		; (redirect errors so they do not mess up output)
;	(shell-command-on-region						; Convert image from ppm to xpm, replacing temp 
;	 (point-min) (point-max) ppmtoxpm-commandline t)			; buffer contents
;	(setq xpm-data (buffer-substring (point-min) (point-max))))		; Save XPM data in string
;      (kill-buffer temp-buffer)							; Delete temp buffer
;      (make-pixmap xpm-data))))							; Create pixmap and return it


(defun hypersrc-convert-any-to-pixmap (filename)
  "Take an arbitrary graphics file FILENAME (relative), convert it to
XPM format if necessary, and return a pixmap object."
  (let (raw-name ppm-converter command-line pixmap current-width)		; Local variables
;;    (setq raw-name (hypersrc-filename-no-suffix filename))
    (setq xpm-name (concat filename ".xpm"))					; XPM file will be created temporarily
    (setq ppm-converter (hypersrc-image-converter-command filename))		; Find converter to use for filename
    (if (not ppm-converter)							; If converter is nil, the file is XPM compatible,
	(make-pixmap (expand-file-name filename))				;   so just create the pixmap from it and return
      (save-excursion								; ELSE:
	(setq current-width (buffer-left-margin-pixwidth))
	(if (< current-width 20) 
	    (error "HyperSource error: Left margin too small."))
	(setenv "HYPERSRC_MARGIN" (- current-width 10))
	(set-buffer (setq temp-buffer (get-buffer-create "*PPM->XPM*")))	; Create temp buffer for status messages
	(setq
	 command-line								; Construct shell command to convert via PPM to XPM
	 (concat
	  ppm-converter " " (expand-file-name filename) " | "
	  hypersrc-ppmscale-command " | "
	  hypersrc-ppmtoxpm-command " > " (expand-file-name xpm-name)))
	(message "Converting %s..." filename)
	(hypersrc-shell-command command-line t)					; Convert image to XPM, write errors to temp buffer
	(message "Converting %s...done." filename))
      (kill-buffer temp-buffer)							; Delete temp buffer
      (setq pixmap (make-pixmap (expand-file-name xpm-name)))			; Create pixmap object
      (delete-file (expand-file-name xpm-name))					; Remove pixmap file
      pixmap)))									; Return pixmap object


(defun hypersrc-convert-any-to-gif (filename width &optional asynchronous)
  "Take an arbitrary graphics file FILENAME (relative), convert it to
GIF format, scaling it to WIDTH, save that file, and return the GIF file name.
Run as subprocess if optional argument ASYNCHRONOUS is non-nil."
  (let (raw-name gif-name ppm-converter scale-command command-line)		; Local variables
    (setq raw-name (hypersrc-filename-no-suffix filename))
    (setq gif-name (concat (file-relative-name raw-name) "-inline.gif"))
    (setq ppm-converter (hypersrc-image-converter-command filename))		; Find converter to PPM for filename
    (save-excursion								; ELSE:
      (if (< (buffer-left-margin-pixwidth) 20) 
	  (error "HyperSource error: Left margin too small."))
      (setenv "HYPERSRC_MARGIN" (- (buffer-left-margin-pixwidth) 10))
      (set-buffer (setq temp-buffer (get-buffer-create "*PPM->GIF*")))		; Create temp buffer for status messages
      (setq
       command-line								; Construct shell command to convert via PPM to GIF
       (concat
	ppm-converter " " (expand-file-name filename) " | "
	hypersrc-ppmscale-command " | "
	hypersrc-ppmtogif-command " > " (expand-file-name gif-name)))
      (if asynchronous (setq command-line (concat command-line " &")))
      (message "Converting %s..." filename)
      (hypersrc-shell-command command-line t)					; Convert image to GIF, write errors to temp buffer
      (message "Converting %s...done." filename))
    (kill-buffer temp-buffer)							; Delete temp buffer
    gif-name))									; Return file name of GIF file just created


;;;; NOT NEEDED ANYMORE
;(defun hypersrc-convert-gif-to-xpm (gif-name)
;  "Use converter programs to create XPM from GIF picture.
;File name of GIF is passed as argument. File name of XPM is returned."
;  (setq raw-name (substring gif-name 0 (string-match "gif$" gif-name)))
;  (setq xpm-name (concat raw-name "xpm"))
;  (start-process-shell-command
;   "gif-converter-process"
;   (setq gif-converter-buffer (generate-new-buffer-name "*GIF -> XPM Converter*"))
;   (concat hypersrc-giftoppm-command " " (expand-file-name gif-name) " | "
;	   hypersrc-ppmtoxpm-command " > " (expand-file-name xpm-name)))
;  (kill-buffer gif-converter-buffer)
;  xpm-name)

;;;; NOT NEEDED ANYMORE
;(defun hypersrc-convert-xpm-to-gif ()
;  "Use converter programs to create GIF from XPM pixmap.
;Pixmap object and name of original file are passed as arguments.
;File name of GIF is returned."
;  (let (raw-name gif-name gif-converter-buffer)
;    (setq raw-name (hypersrc-filename-no-suffix (expand-file-name target-name)))
;    (setq gif-name (concat raw-name ".gif"))
;    (message "Converting image from XPM to GIF...")
;    (save-excursion
;      (set-buffer (setq gif-converter-buffer (generate-new-buffer-name "*XPM -> GIF Converter*")))
;    (shell-command
;     (concat hypersrc-xpmtoppm-command " " (expand-file-name xpm-name) " | "
;	     hypersrc-ppmtogif-command " > " (expand-file-name gif-name)) t))
;    (kill-buffer gif-converter-buffer)
;    gif-name))

(provide 'hypersrc-graphics)
