;;; hypersrc-menus.el -- Mode-specific menu and functions to add/remove/show it

;;; $Id: hypersrc-menus.el,v 0.3 1995/05/17 09:17:51 job Exp $

(defvar hypersrc-menu
  '("HyperSource"
    ["Create/Edit Header" hypersrc-edit-create-header-from-region 
     :active t :keys "C-c C-h"]
    ["Create/Edit Margin Comment" hypersrc-edit-create-marginal-comment 
     :active t :keys "C-c C-;"]
    ["Add Margin Image/TeX Formula" hypersrc-create-margin-image 
     :active t :keys "C-c C-i"]
    ["Add Link" hypersrc-create-explicit-link 
     :active t :keys "C-c C-l"]
    ["Follow Link" hypersrc-follow-explicit-link 
     :active t :keys "M-button1"]
    ["Delete Link" hypersrc-delete-explicit-link 
     :active t :keys "M-button2"]
    ["Save as Plain Text" hypersrc-save-buffer-no-format 
     :active t :keys "C-c C-s"]
    ["Change Margin Width" hypersrc-set-margin-width
     :active t :keys "C-c C-m"]
    )
  "*Menu added to menubar, and as popup on button3 while in HyperSource mode.
Call 'hypersrc-remove-menu-from-menubar' or
'hypersrc-remove-menu-from-popup' if you do not want it at one of these
places.")

(defun hypersrc-add-menu-to-menubar ()
  "Add HyperSource menu to menubar."
  (interactive)
  (if (and current-menubar (not (assoc (car hypersrc-menu) current-menubar)))
      (progn
	(add-menu nil (car hypersrc-menu) (cdr hypersrc-menu))
	(set-menubar-dirty-flag))))

(defun hypersrc-remove-menu-from-menubar ()
  "Remove HyperSource menu from menubar."
  (interactive)
  (if (and current-menubar (assoc (car hypersrc-menu) current-menubar))
      (progn
	(delete-menu-item (list (car hypersrc-menu)))
	(set-menubar-dirty-flag))))

(defun hypersrc-add-menu-to-popup ()
  "Put HyperSource poup menu on right mouse button."
  (interactive)
  (define-key hypersrc-mode-map '[(button3)] 'hypersrc-popup-menu))

(defun hypersrc-remove-menu-from-popup ()
  "Remove HyperSource poup menu from right mouse button."
  (interactive)
  (define-key hypersrc-mode-map '[(button3)] nil))

(defun hypersrc-popup-menu (event)
  "Pop up HyperSource mode menu."
  (interactive "@e")
  (popup-menu hypersrc-menu))


(provide 'hypersrc-menus)
