;;; hypersrc-vars.el -- User variables for HyperSource mode

;;; $Id: hypersrc-vars.el,v 0.2 1995/05/23 12:54:39 job Exp $

(defvar hypersrc-left-margin-width 20
  "*Width of left margin in HyperSource mode.
The margin is used for annotations, i.e., comments and pixmaps.")
(make-variable-buffer-local 'hypersrc-left-margin-width)

(defvar hypersrc-ppmtoxpm-command "ppmtoxpm"
  "*Command to convert a PPM image to XPM format.
It should read from stdin, and write to stdout.
See documentation of hypersrc-image-type-alist for more information.")

(defvar hypersrc-ppmtogif-command "ppmtogif"
  "*Command to convert a PPM image to GIF format.
It should read from stdin, and write to stdout.
See documentation of hypersrc-image-type-alist for more information.")

(defvar hypersrc-ppmscale-command "pnmscale.sh"
  "*Command to shrink a PPM image so that it is no wider than
$HYPERSRC_MARGIN.")

(defvar hypersrc-image-type-alist
  (cond
   ((eq system-type 'berkeley-unix)	; Settings for Suns
    '(
     ("xpm" "pixmap -f" "xpmtoppm")
     ("xbm" "pixmap -f" "xbmtopbm")
     ("idraw" "idraw.sh" "pstoppm.sh")
     ("ps" "idraw.sh" "pstoppm.sh")
     ("gif" "xpaint" "giftoppm")
     ("tiff" "xpaint" "tifftopnm")
     ("jpg" nil "jpegtopnm")
     ("jpeg" nil "jpegtopnm")
     ("fml" t "textoppm.sh")
     ))
   ((eq system-type 'irix)		; Settings for SGI machines
    '(
     ("xpm" "lindon pixmap -f" "xpmtoppm")
     ("xbm" "lindon pixmap -f" "xbmtopbm")
     ("idraw" "lindon idraw.sh" "lindon pstoppm.sh")
     ("ps" "lindon idraw.sh" "lindon pstoppm.sh")
     ("gif" "lindon xpaint" "giftoppm")
     ("tiff" "lindon xpaint" "lindon tifftopnm")
     ("jpg" nil "lindon jpegtopnm")
     ("jpeg" nil "lindon jpegtopnm")
     ("fml" t "lindon textoppm.sh")
     ))
   (t					; Default settings for other systems
    '(
     ("xpm" "pixmap -f" "xpmtoppm")
     ("xbm" "pixmap -f" "xbmtopbm")
     ("idraw" "idraw.sh" "pstoppm.sh")
     ("ps" "idraw.sh" "pstoppm.sh")
     ("gif" "xpaint" "giftoppm")
     ("tiff" "xpaint" "tifftopnm")
     ("jpg" nil "jpegtopnm")
     ("jpeg" nil "jpegtopnm")
     ))
   )
  "*List of supported image file formats for HyperSource mode.

Each file format is represented by a sublist of the form
(SUFFIX EDITOR CONVERTER). It contains a file suffix, the name of a
graphics editor program that understands this format, and the name of a
converter program that can translate this format into the Portable
Pixmap format (PPM), as defined by the pbmplus or netpbm
package. Converters should read a filename as arg, and write to stdout.

If EDITOR is 'nil, no editor is available.
If EDITOR is 't, the file is loaded into XEmacs as text file.

You may have to edit or add entries to this list to use other image
formats (XPM is the X Pixmap format supported by XEmacs).")


(defvar hypersrc-mode-comment-alist
  '((c-mode    "/* " " */")
    (c++-mode  "/* " " */")
    (objc-mode "/* " " */")
    (pascal-mode "(* " " *)"))
  "*List of lists, each with a major mode symbol and its comment delimiters.
  HyperSource minor mode uses this list to determine which comment
delimiters to insert when writing HTML files.
  To enable HyperSource mode for your favourite 'Foo' programming language
using a mode called 'foo-mode', add the following to your .emacs:

(setq hypersrc-mode-comment-alist
      (append hypersrc-mode-comment-alist
              (list (list foo-mode "{* " " *}"))))
(add-hook 'foo-mode-hook 'hypersrc-loading-mode)

  The language must have open and close comment sequences ( '{*' and '*}
in this example).  Don't forget the spaces inside the delimiters.
  See the file 'hypersrc-vars.el' for details.")

;; Make some major modes call a function that switches on
;; HyperSource mode if the file is in HyperSource format:
;;
(add-hook 'c-mode-hook 'hypersrc-loading-mode)
(add-hook 'c++-mode-hook 'hypersrc-loading-mode)
(add-hook 'objc-mode-hook 'hypersrc-loading-mode)
(add-hook 'pascal-mode-hook 'hypersrc-loading-mode)

;;; Load .xyz.hs files with the major mode for .xyz files:
;;; THIS IS DONE AUTOMAGICALLY NOW BY CHECKING AUTO-MODE-ALIST!
;;; (see hypersrc.el)

(provide 'hypersrc-vars)
