;;; hypersrc.el --- minor mode to edit HyperSource files

;; Copyright (C) 1995 Jan Oliver Borchers

;; Author:   Jan O. Borchers <job@ira.uka.de>
;; Created:  07 Mar 1995
;; RCS Info: $Id: hypersrc.el,v 0.8 1995/05/23 15:18:05 job Exp $
;; Keywords: hypertext, html, c, programming, documentation, xemacs


;; INTRODUCTION
;;
;; This package provides the HyperSource minor mode to edit
;; HyperSource files. It uses inline images, and thus currently
;; requires XEmacs (19.11 or later) to run.
;;
;; It provides a menu to insert margin images, formulae and comments,
;; as well as headers and links, into your source files in a WYSIWYG style. 
;; This makes your programs easier to read, understand, and reuse.
;;
;; Files are saved as HTML 3.0 documents and can be browsed by others using
;; any Viewer that understands HTML 3.0 (I suggest using Netscape 1.1 or
;; later).
;;
;; Compilable source code can be extracted via a menu entry inside XEmacs,
;; or by using the "Save As ASCII" option of your browser.


;; WHAT YOU NEED
;;
;; To use HyperSource, you will need:
;; - XEmacs 19.11 or later
;; - Converters to the PPM image format for all graphics formats you wish
;;   to use. I recommend using the netpbm package, available from:
;;     * wuarchive.wustl.edu (128.252.135.4), in /graphics/graphics/packages/NetPBM
;;     * ikaros.fysik4.kth.se (130.237.35.2), in /pub/netpbm
;;     * ftp.informatik.uni-oldenburg.de (134.106.1.9)
;;     * peipa.essex.ac.uk (155.245.115.161), in ipa/src/manip
;;     * ftp.rahul.net (192.160.13.1), in /pub/davidsen/source
;;     * ftp.cs.ubc.ca, in /ftp/archive/netpbm
;; - Graphics editors for the formats you wish to edit. The system is
;;   preconfigured to use idraw for PostScript drawings, and xpaint for
;;   raster images.
;; - The LaTeX system, if you wish to insert formulae or other
;;   arbitrarily formatted text into your source code.


;; INSTALLATION
;;
;; - Move all the "hypersrc*.el" files to ".../lib/xemacs/site-lisp/".
;; - Move all the ".sh" shell scripts to a directory in your PATH variable.
;; - Add the following to your .emacs file:
;;   (require 'hypersrc)
;; - Install the packages you want & don't have (see "What You Need" above).


;; USAGE INSTRUCTIONS
;;
;; - Load a C, C++, or Objective C file.
;; - "M-x hypersrc-mode" switches HyperSource on or off.
;;   A menu will appear in the menu bar, and as third mouse button popup menu.
;; - To insert drawings, choose "Add Margin Image/Tex Formula" and give
;;   the name of an image file. The editor is called depending on the fil
;;   name suffix (idraw for .ps files, etc.)
;; - To insert a formula, choose the same menu, but give a filename
;;   ending in .fml. A new buffer pops up in which you can enter
;;   arbitrary LaTeX "body" text (ie, without the preamble).
;;   For example, just write "The equation is:\\ $f(x)=x^2+3x+2$" to
;;   create that text & formula.
;; - If something doesn't work, or to use HyperSource with other languages,
;;   check "hypersrc-vars.el" and the shell scripts for customization.


;; FILES
;;
;; This file is part of the HyperSource package, which consists of the
;; following files:
;;
;; - hypersrc.el          -- Main init, editing, and navigation functions
;; - hypersrc-vars.el     -- All customizable variables HyperSource uses  <---
;; - hypersrc-menus.el    -- Menu functions and definitions
;; - hypersrc-faces.el    -- Face definitions (fonts and colours)
;; - hypersrc-read.el     -- File loading functions to parse HTML
;; - hypersrc-write.el    -- File saving functions to create HTML
;; - hypersrc-ids.el      -- Create unique identifiers (used for link targets)
;; - hypersrc-graphics.el -- Functions to convert between XPM and GIF
;; - hypersrc-comments.el -- Find comment delimiters depending on mode


;; COPYING
;;
;; HyperSource is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 1, or (at your option)
;; any later version.
;;
;; HyperSource is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with HyperSource; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.


;; --- NO USER SERVICEABLE PARTS BELOW THIS LINE ---

;; CODE

(setq hypersrc-mode-alist nil)
(mapcar
 (function
  (lambda (cell)
    (cond
     ((string-match (regexp-quote "$") (car cell))
      (if
	  (and
	   (not
	    (assoc
	     (concat
	      (substring (car cell) 0 (string-match (regexp-quote "$") (car cell)))
	      ".hs$")
	     auto-mode-alist))
	   (not
	    (string-match (regexp-quote ".hs$") (car cell))))
	  (setq
	   hypersrc-mode-alist
	   (append
	    hypersrc-mode-alist
	    (list
	     (cons
	      (concat
	       (substring (car cell) 0 (string-match (regexp-quote "$") (car cell)))
	       ".hs$")
	      (cdr cell)))))))
     ((string-match (regexp-quote "\\'") (car cell))
      (if
	  (and
	   (not
	    (assoc
	     (concat
	      (substring (car cell) 0 (string-match (regexp-quote "\\'") (car cell)))
	      ".hs\\'")
	     auto-mode-alist))
	   (not
	    (string-match (regexp-quote ".hs\\'") (car cell))))
	  (setq
	   hypersrc-mode-alist
	   (append
	    hypersrc-mode-alist
	    (list
	     (cons
	      (concat
	       (substring (car cell) 0 (string-match (regexp-quote "\\'") (car cell)))
	       ".hs\\'")
	      (cdr cell))))))))
    nil))
 auto-mode-alist)
(setq auto-mode-alist (append auto-mode-alist hypersrc-mode-alist))

;;; Don't need this for our minor mode!
;;; (defvar hypersrc-mode-abbrev-table nil
;;;   "Abbrev table used while in HyperSource mode.")
;;; (define-abbrev-table 'hypersrc-mode-abbrev-table ())
;;;
;;; (defvar hypersrc-mode-syntax-table nil
;;;   "Syntax table used while in HyperSource mode.")

;; Load annotations package (in lisp/utils/) for marginal annotations:
;;
(require 'annotations)

;;; Load menu-related definitions:
;;;
(require 'hypersrc-menus)

;;; Load face definitions (fonts, colours, etc.):
;;;
(require 'hypersrc-faces)

;;; Load ID generator code for unique link target identification:
;;;
(require 'hypersrc-ids)

;;; Load file i/o routines (loading and saving HyperSource files):
;;;
(require 'hypersrc-read)
(require 'hypersrc-write)

;;; Load user variables:
;;;
(require 'hypersrc-vars)


(makunbound 'hypersrc-process-annotation-alist)
(defvar hypersrc-process-annotation-alist ()
  "Association list mapping names of running editor
subprocesses to their annotation.")
(make-variable-buffer-local 'hypersrc-process-annotation-alist)

;; Save left margin width (if buffer-left-margin-width returns nil,
;; this means "use screen value"):
;; REPLACE screen with frame for 19.12!
(defvar hypersrc-original-left-margin-width
  (or (buffer-left-margin-width) (screen-left-margin-width) 0)
  "Left margin of current buffer before HyperSource mode was enabled.")
(make-variable-buffer-local 'hypersrc-original-left-margin-width)

(defvar hypersrc-mode-map nil
  "Keymap for HyperSource mode.")

(if hypersrc-mode-map
    ()
  (setq hypersrc-mode-map (make-sparse-keymap))
  (set-keymap-name hypersrc-mode-map 'hypersrc-mode-map))

(define-key hypersrc-mode-map
  '[(control c) (control h)] 'hypersrc-edit-create-header-from-region)
(define-key hypersrc-mode-map
  '[(control c) (control \;)] 'hypersrc-edit-create-marginal-comment)
(define-key hypersrc-mode-map
  '[(control c) (control i)] 'hypersrc-create-margin-image)
(define-key hypersrc-mode-map
  '[(control c) (control l)] 'hypersrc-create-explicit-link)
(define-key hypersrc-mode-map
  '[(control c) (control s)] 'hypersrc-save-buffer-no-format)
(define-key hypersrc-mode-map
  '[(control c) (control m)] 'hypersrc-set-margin-width)
(define-key hypersrc-mode-map
  '[(meta button1)] 'hypersrc-follow-implicit-link)
(define-key hypersrc-mode-map '[(button3)] 'hypersrc-popup-menu)

(defvar hypersrc-link-map nil
  "Keymap valid while mouse is over a HyperSource link extent.")

(if hypersrc-link-map
    ()
  (setq hypersrc-link-map (make-sparse-keymap))
  (set-keymap-name hypersrc-link-map 'hypersrc-link-map))

(define-key hypersrc-link-map
  '[(meta button1)] 'hypersrc-follow-explicit-link-under-mouse)
(define-key hypersrc-link-map
  '[(meta button2)] 'hypersrc-delete-explicit-link-under-mouse)


(or (assq 'hypersrc-mode minor-mode-map-alist)
    (setq minor-mode-map-alist
	  (cons (cons 'hypersrc-mode hypersrc-mode-map)
		minor-mode-map-alist)))

;;; Making some variables local to each buffer:
;;; Do we have to set a default value?

(defvar hypersrc-mode nil
  "Non-nil means using HyperSource minor mode (usually with C/C++ mode).")

(make-variable-buffer-local 'hypersrc-mode)
(put 'hypersrc-mode 'permanent-local t)
(or (assq 'hypersrc-mode minor-mode-alist)
    (setq minor-mode-alist
	  (cons '(hypersrc-mode " Hyper") minor-mode-alist)))

(defun hypersrc-mode (&optional arg)
  "Toggle HyperSource mode.
With numerical ARG, turn HyperSource minor mode on if ARG is positive,
off otherwise.

It provides a menu to insert margin images, formulae and comments,
as well as headers and links, into your source files in a WYSIWYG style.
This makes your programs easier to read, understand, and reuse.

Files are saved as HTML 3.0 documents and can be browsed by others using
any Viewer that understands HTML 3.0 (I suggest using Netscape 1.1 or
later, available from ftp.netscape.com, in /netscape/unix/).

Compilable source code can be extracted via a menu entry inside XEmacs,
or by using the \"Save As ASCII\" option of your browser.

To insert drawings, choose \"Add Margin Image/Tex Formula\" and give
the name of an image file. The editor is called depending on the fil
name suffix (idraw for .ps files, etc.)

To insert a formula, choose the same menu, but give a filename
ending in .fml. A new buffer pops up in which you can enter
arbitrary LaTeX \"body\" text (ie, without the preamble).
For example, just write \"The equation is:\\ $f(x)=x^2+3x+2$\" to
create that text & formula. Leave that buffer with C-c C-c.

Note: The converters need about 10 seconds to create the images.

If something doesn't work, or to use HyperSource with other languages,
check \"hypersrc-vars.el\" and the shell scripts for customization.

See the beginning of the file \"hypersrc.el\" for more details.

Special commands:\\{hypersrc-mode-map}
Turning on HyperSource mode calls the value of the variable
`hypersrc-mode-hook',if that value is non-nil."
  (interactive "P")
  (setq hypersrc-mode
	(cond ((null arg) (not hypersrc-mode))
	      ((numberp arg) (> (prefix-numeric-value arg) 0))
	      (t t)))
  (force-mode-line-update)
  (if hypersrc-mode
      (progn
	;; Switch HyperSource mode ON:
	
	;; If the file contains HyperSource (HTML) format information,
	;; parse it:
	(if (save-excursion
	      (search-forward "<!-- HyperSource 1.0 File Format -->" nil t))
	    (hypersrc-parse-buffer)
	  ;; else: File doesn't contain HTML markup yet
	  (set-visited-file-name (concat buffer-file-name ".hs"))
	  ;; Set buffer margin for annotations:
	  (setq hypersrc-original-left-margin-width (buffer-left-margin-width))
	  (set-buffer-left-margin-width hypersrc-left-margin-width)
	  ;; Should we make this restorable?
	  (setq use-left-overflow t))

	  ;; In either case:
	  
	  ;; Resize window accordingly: Doesn't work with single
	  ;; window. Should be left to the user anyway. (?)
	  ;; (enlarge-window-horizontally 
	  ;;  (- hypersrc-left-margin-width hypersrc-original-left-margin-width))
	  
	  ;; Install menu...
	  (hypersrc-add-menu-to-menubar)
	  ;; ...and replace standard write-file function for *this* buffer:
	  (or (memq 'hypersrc-save-buffer local-write-file-hooks)
	      (setq local-write-file-hooks
		    (cons 'hypersrc-save-buffer local-write-file-hooks)))
	  
	  ;; (setq local-abbrev-table hypersrc-mode-abbrev-table)
	  ;; (set-syntax-table hypersrc-mode-syntax-table)
	  
	  (run-hooks 'hypersrc-mode-hook))
    
    ;; Else, switch HyperSource mode OFF:
    ;; Remove special hook in write-file to save HyperSource files:
    (if (memq 'hypersrc-save-buffer local-write-file-hooks)
	(setq local-write-file-hooks
	      (delq 'hypersrc-save-buffer local-write-file-hooks)))
    ;; Restore buffer left margin and window size:
    (set-buffer-left-margin-width hypersrc-original-left-margin-width)
    ;; Remove menu:
    (hypersrc-remove-menu-from-menubar)
    ;; Remove possible .hs extension from the name of the visited file:
    (set-visited-file-name
     (substring buffer-file-name 0 (string-match "\\.hs$" buffer-file-name)))
 
    ;; leave this to user:
    ;; (shrink-window-horizontally
    ;;  (- hypersrc-left-margin-width hypersrc-original-left-margin-width))
))


(defun hypersrc-loading-mode ()
  "This function is hooked into the major mode hooks to enable HyperSource mode.
If the file just loaded is a HyperSource file, we parse it, and enable
HyperSource minor mode. Otherwise, we do nothing so raw C files etc. can
still be edited as usual."

  ;; If file is in HyperSource format:
  (if (save-excursion
	(search-forward "<!-- HyperSource 1.0 File Format -->" nil t))
      ;; Parse the markup information in it:
      (progn
	(hypersrc-parse-buffer)
	;; Install HyperSource menubar:
	(setq hypersrc-mode t)
	(force-mode-line-update)
	(hypersrc-add-menu-to-menubar)
	;; Replace standard write-file function for *this* buffer:
	(or (memq 'hypersrc-save-buffer local-write-file-hooks)
	    (setq local-write-file-hooks
		  (cons 'hypersrc-save-buffer local-write-file-hooks)))
	(run-hooks 'hypersrc-mode-hook))))


(defun hypersrc-edit-create-header-from-region (from to size)
  "Create, edit, or remove a header with region as text.
Prompts for size (1=biggest - 6=smallest, 0=remove, default 1)."
  (interactive
   "r\nsHeader size (1=biggest - 6=smallest, 0=remove, default 1): ")
  (let (header)									; Local variables
    ;; Find existing header for region, or create new one:
    (setq header (or (extent-at from nil 'headersize) (make-extent from to)))
    ;; Override font-lock face settings of other extents, e.g., those
    ;; created by font-lock:
    (set-extent-priority header 1)
    ;; Make extent being copied when its text is killed, yanked, undone, etc.:
    (set-extent-property header 'duplicable t)
    ;; If user just hits return when prompted for size, it is set to "",
    ;; so choose 1 as default:
    (setq size (if (string= size "") 1 (string-to-int size)))
    (if (< size 0) (setq size 0))
    (if (> size 6) (setq size 6))
    (set-extent-property header 'hypersrc 'header)
    (set-extent-property header 'headersize size)
    (cond
     ((eq size 0) (delete-extent header))
     ((eq size 1) (set-extent-face header 'hypersrc-header1-face))
     ((eq size 2) (set-extent-face header 'hypersrc-header2-face))
     ((eq size 3) (set-extent-face header 'hypersrc-header3-face))
     ((eq size 4) (set-extent-face header 'hypersrc-header4-face))
     ((eq size 5) (set-extent-face header 'hypersrc-header5-face))
     ((eq size 6) (set-extent-face header 'hypersrc-header6-face)))
    ;; Header extent has been inserted, so buffer is now modified:
    (set-buffer-modified-p t)))


(defun hypersrc-edit-marginal-comment-with-mouse (annotation)
  "Edit a comment (textual annotation) in margin of current line.
Invoked by clicking button1 over an existing comment in margin."
  (set-annotation-glyph
   annotation (read-string "Comment (empty to delete): " (annotation-glyph annotation)))
  ;; Comment extent has been edited, so buffer is now modified:
  (if (string= (annotation-glyph annotation) "") (delete-annotation annotation))
  (set-buffer-modified-p t))


(defun hypersrc-first-non-nil-elt (list)
  "Return the first element of LIST that is non-nil, or nil otherwise."
  (if (null list)
      nil
    (or (car list) (hypersrc-first-non-nil-elt (cdr list)))))


(defun hypersrc-one-annotation-at (&optional pos) 
  "Return first annotation with length 1 at POS, or at point if POS=nil."
  (let (possible-annotations matching-annotations)				; Local variables
    ;; if no pos specified, set it to point:
    (setq pos (or pos (point)))
    ;; get list of all annotations overlapping pos or the char before it:
    (setq possible-annotations (annotations-in-region pos (1+ pos) (current-buffer)))
    ;; build a list containing all annotations (usually just one) at pos:
    (setq matching-annotations
	  (mapcar
	   (function
	    (lambda (annotation)
	      (if (and (annotationp annotation)
		       (eq (extent-start-position annotation) pos)
		       (eq (extent-length annotation) 1))
		  annotation
		nil)))
	   possible-annotations))
    ;; return first annotation found, or nil:
    (hypersrc-first-non-nil-elt matching-annotations)))

(defun hypersrc-create-new-marginal-comment ()
  "Create new comment (textual annotation) at point.
Returns the newly created annotation."
  (let (current-annotation)							; Local variables
    (setq current-annotation (make-annotation "" nil 'outside-margin))
    (set-annotation-face current-annotation 'hypersrc-comment-face)
    (set-annotation-data current-annotation current-annotation)
    (set-annotation-action
     current-annotation 'hypersrc-edit-marginal-comment-with-mouse)
    (set-extent-property current-annotation 'duplicable t)
    ;; All HyperSource extents are marked with this property:
    (set-extent-property current-annotation 'hypersrc 'comment)
    (set-extent-priority current-annotation 1)
    current-annotation))


(defun hypersrc-edit-create-marginal-comment ()
  "Edit or create a comment (textual annotation) in margin of current line."
  (interactive)
  (let (current-annotation)							; Local variables
    (save-excursion
      (beginning-of-line)
      ;; If there is an annotation at the beginning of the current line,
      ;; use it; otherwise create a new one:
      (if (hypersrc-one-annotation-at)
	  (setq current-annotation (hypersrc-one-annotation-at))
	(progn
	  (setq current-annotation (hypersrc-create-new-marginal-comment))
	  (set-extent-endpoints current-annotation
				(extent-start-position current-annotation)
				(1+ (extent-end-position current-annotation))))))
    ;; Read new comment text:
    (set-annotation-glyph
     current-annotation
     (read-string "Comment (empty to delete): "
		  (annotation-glyph current-annotation)))
    (if (string= (annotation-glyph current-annotation) "")
	(delete-annotation current-annotation))
  ;; Comment extent has been created/edited/deleted, so buffer is modified:
  (set-buffer-modified-p t)))


(defun hypersrc-setup-margin-image (annotation target-name)
  "Set the properties, etc., of image annotation ANNOTATION
pointing to TARGET-NAME."
  (set-extent-property annotation 'target target-name)
  (set-annotation-layout annotation 'outside-margin)
  (set-annotation-action annotation 'hypersrc-pixmap-action)
  ;; The DATA field contains a cons cell with the annotation itself
  ;; and the corresponding target file name in it.
  (set-annotation-data annotation (cons annotation target-name))
  ;; All HyperSource extents are marked with this property:
  (set-extent-property annotation 'hypersrc 'image)
  (set-extent-priority annotation 1)
  (set-extent-property annotation 'duplicable t))


;; Needed by the following two functions to communicate:
;;
(setq hypersrc-current-annotation nil)

(defun hypersrc-create-margin-image (&optional pos) 
  "Create image in left margin of line containing POS.
POS defaults to point. An annotation consists of an inline image and a
reference to a target image. The target image can be in any format that
can be converted to PPM (see the documentation of
hypersrc-image-type-alist for details).  The inline image serves as a
preview and link to the target file. Prompts for file name of target
image in minibuffer. If the target file does not exist, the appropriate
editor is started to create it.  The inline image is created
automatically as a preview from it."
  (interactive)
  (let (new-annotation target-name new-process)					; Local variables
    (save-excursion
      (if pos (goto-char pos))
      (beginning-of-line)
      ;; Get target image name:
      (setq
       target-name
       (file-relative-name 
	(read-file-name "Target image file (or .fml LaTeX formula file): ")))
      (if (not (file-exists-p target-name))
	  (if (y-or-n-p-minibuf 
	       (format "File %s doesn't exist, create? " target-name))
	      (if (hypersrc-image-editor-command target-name)
		  ;; External editor known for this type of file
		  (progn
		    ;; Create annotation at point in current buffer:
		    (setq new-annotation (make-annotation ""))
		    (hypersrc-setup-margin-image new-annotation target-name)
		    ;; Give annotation a length of 1,
		    ;; so that it can be copied/yanked, etc.:
		    (set-extent-endpoints new-annotation
					  (extent-start-position new-annotation)
					  (1+ (extent-end-position new-annotation)))
		    (if (eq (hypersrc-image-editor-command target-name) t)
			;; editor=t means create file in Emacs buffer:
			(progn
			  (setq hypersrc-current-window (selected-window))
			  (find-file-other-window target-name)
			  (setq hypersrc-current-annotation new-annotation)
			  (local-set-key '[(control c) (control c)]
					 'hypersrc-end-edit-image-buffer)
			  (message
			   (substitute-command-keys
			    "Press C-c C-c when finished.")))
		      ;; Else: Start external editor...
		      (message "Spawning external image editor...")
		      (setq new-process
			    (start-process-shell-command
			     (concat "image-editor-process-" target-name)
			     ;; start name with space for invisibility?
			     (generate-new-buffer-name "*Image Editor*")
			     (concat 
			      (hypersrc-image-editor-command target-name)
			      " " (expand-file-name target-name))))
		      (message "Spawning external image editor...done.")
		      (set-process-sentinel
		       new-process 'hypersrc-image-editor-process-end-sentinel)
		      (setq hypersrc-process-annotation-alist
			    (cons
			     (cons (process-name new-process) new-annotation)
			     hypersrc-process-annotation-alist))))
		;; Else: Error - no external editor found
		(error 
		 "HyperSource error: No editor for \"%s\" in hypersrc-image-type-alist"
		 hypersrc-filename-suffix target-name))
	    ;; Else: User entered "n" -> don't do anything
	    (error "HyperSource: Aborted."))
	;; Else: File exists
	;; Convert target file to XPM format, allocate new pixmap in X server,
	;; and create annotation with it:
	;; This creates error message automatically if no converter was found
	(setq new-annotation
	      (make-annotation (hypersrc-convert-any-to-pixmap target-name)))
	(hypersrc-setup-margin-image new-annotation target-name)
	;; Give annotation a length of 1,
	;; so that it can be copied/yanked, etc.:
	(set-extent-endpoints new-annotation
			      (extent-start-position new-annotation)
			      (1+ (extent-end-position new-annotation)))
	(redraw-screen (selected-screen))
	;; Pixmap extent has been created, so buffer is now modified:
	(set-buffer-modified-p t)))))


(defun hypersrc-end-edit-image-buffer ()
  "Finish editing a target image file in a buffer.
Triggers saving and conversion of the file to create XPM image."
  (interactive)
  (save-buffer)
  (delete-window)
  (select-window hypersrc-current-window)
  (set-annotation-glyph
   hypersrc-current-annotation
   (hypersrc-convert-any-to-pixmap
    (cdr (annotation-data hypersrc-current-annotation))))
  (redraw-screen (selected-screen)))


(defun hypersrc-image-editor-process-end-sentinel (finished-process event)
  (let (annotation filename pixmap)						; Local variables
    (message "Running sentinel...")
    ;; find annotation and its filename in the alist:
    (setq annotation 
	  (cdr
	   (assoc
	    (process-name finished-process) hypersrc-process-annotation-alist)))
    (setq filename (cdr (annotation-data annotation)))
    ;; Re-build pixmap from file (in case it was changed), and redraw screen:
    (force-redisplay)
    (setq pixmap (hypersrc-convert-any-to-pixmap filename))
    (set-annotation-glyph annotation pixmap)
    (force-redisplay)
    ;; remove this process-annotation pair from the alist:
    (delete
     (assoc (process-name finished-process) hypersrc-process-annotation-alist)
     hypersrc-process-annotation-alist)
    ;; Image has been edited, so buffer should be considered modified:
    (set-buffer-modified-p t)
    (message "Running sentinel...done.")))


(defun hypersrc-pixmap-action (annotation-data)
  "Action callback when a margin pixmap is selected.
User can choose D to delete image from buffer, or E to start appropriate
editor for the original image file (GIF, JPEG, PostScript,...) as a
subprocess. The editor to use is selected from 'hypersrc-image-type-alist',
depending on the file suffix (.gif, .jpg, .ps,...).
See documentation of that variable for more information."
  (let (annotation filename user-input new-process)				; Local variables
    (setq annotation (car annotation-data)
	  filename (cdr annotation-data))
    (setq user-input (downcase (read-string "E)dit image (default) or D)elete? " "E")))
    (cond
     ((string= user-input "d")
      (delete-annotation annotation))
     (t
      (if (hypersrc-image-editor-command filename)
	  (if (eq (hypersrc-image-editor-command filename) t)
	      ;; editor=t means create file in Emacs buffer:
	      (progn
		(setq hypersrc-current-window (selected-window))
		(find-file-other-window filename)
		(setq hypersrc-current-annotation annotation)
		(local-set-key '[(control c) (control c)]
			       'hypersrc-end-edit-image-buffer)
		(message
		 (substitute-command-keys
		  "Press C-c C-c when finished.")))
	    ;; Else: Start external editor...
	    (message "Spawning external image editor...")
	    (setq new-process
		  (start-process-shell-command
		   (concat "image-editor-process-" filename)
		   ;; start name with space for invisibility?
		   (generate-new-buffer-name "*Image Editor*")
		   (concat (hypersrc-image-editor-command filename) " " filename)))
	    (message "Spawning external image editor...done.")
	    (set-process-sentinel
	     new-process 'hypersrc-image-editor-process-end-sentinel)
	    (setq hypersrc-process-annotation-alist
		  (cons
		   (cons (process-name new-process) annotation)
		   hypersrc-process-annotation-alist)))
	;; Else: Error - don't know what editor to use!
	(error
	 "HyperSource error: Please add editor name for \"%s\" files to hypersrc-image-type-alist"
	 (hypersrc-filename-suffix filename)))))))


;; Needed by the following two functions to communicate:
(setq hypersrc-current-link nil)
(setq hypersrc-current-window nil)
(setq hypersrc-current-dir nil)

(defun hypersrc-create-explicit-link (from to)
  "Create an explicit link from region."
  (interactive "r")
  (let (target-file-name)							; Local variables
  
    ;; This was for finding the word under point. Not needed anymore since
    ;; we now create links from the active region:
;  (save-excursion
;    (skip-syntax-backward "w_")
;    (setq beginning-of-symbol (point))
;    (skip-syntax-forward "w_")
;    (setq end-of-symbol (point)))
;  (setq hypersrc-current-link (make-extent beginning-of-symbol end-of-symbol))

    (setq hypersrc-current-link (make-extent from to))
    (setq hypersrc-current-dir default-directory)
    (set-extent-face hypersrc-current-link 'hypersrc-link-face)
    (set-extent-property hypersrc-current-link 'keymap hypersrc-link-map)
    (setq target-file-name (file-relative-name (read-file-name "Target file: ")))
    ;; All HyperSource extents are marked with this property:
    (set-extent-property hypersrc-current-link 'hypersrc 'link)
    (set-extent-priority hypersrc-current-link 1)
    (set-extent-property hypersrc-current-link 'duplicable t)
    (setq hypersrc-current-window (selected-window))
    ;; Link extent has been inserted, so buffer is now modified:
    (set-buffer-modified-p t)
    (find-file-other-window target-file-name)
    ;; (view-mode)
    ;; If target is a hypersource file, ask for exact target region.
    ;; Otherwise, just leave the file open.
    (if hypersrc-mode
	(progn
	  (local-set-key '[(control c) (control c)] 'hypersrc-quit-target-file)
	  (message
	   (substitute-command-keys
	    "Mark target region, then press C-c C-c")))
      (set-extent-property
       hypersrc-current-link 'target-file target-file-name))))
    

(defun hypersrc-quit-target-file (from to)
  "Return region to define position in a file for a link target."
  (interactive "r")
  (let (id new-target)								; Local variables
    (local-unset-key '[(control c) (control c)])
    (set-extent-property
     hypersrc-current-link 'target-file
     (file-relative-name buffer-file-name hypersrc-current-dir))
    ;; (view-quit)
    (if hypersrc-mode
	(progn
	  (setq id (concat (buffer-substring from to)
			   "[" (int-to-string (hypersrc-new-id)) "]"))
	  (setq new-target (make-extent from to))
	  (set-extent-property new-target 'target-id id)
	  (set-extent-face new-target 'hypersrc-target-face)
	  ;; (set-extent-property target 'keymap hypersrc-link-map)
	  ;; All HyperSource extents are marked with this property:
	  (set-extent-property new-target 'hypersrc 'target)
	  (set-extent-priority new-target 1)
	  (set-extent-property new-target 'duplicable t)
	  ;; Target extent has been inserted, so buffer is now modified:
	  (set-buffer-modified-p t)
	  (select-window hypersrc-current-window)
	  ;; (set-extent-property hypersrc-current-link 'target-start from)
	  ;; (set-extent-property hypersrc-current-link 'target-end to)
	  (set-extent-property hypersrc-current-link 'target-id id)))))


(defun hypersrc-follow-explicit-link ()
  "Follow the explicit link under point.
If the target is a HyperSource file, move to the exact target position.
Otherwise, just find the target file."
  (interactive)
  (let (link id-to-find)							; Local variables
    (setq link (extent-at (point) nil 'target-file))
    (find-file-other-window (expand-file-name (extent-property link 'target-file)))
    (setq id-to-find (extent-property link 'target-id))
    (if id-to-find
	(goto-char
	 (map-extents
	  (function
	   (lambda
	     (extent maparg)
	     (if
		 (and
		  (eq (extent-property extent 'hypersrc) 'target)    ; extent is hypersrc target
		  (string= (extent-property extent 'target-id) id-to-find)) ; right ID
		 (extent-start-position extent)))))))))


(defun hypersrc-follow-explicit-link-under-mouse (event)
    "Follow the explicit link under the mouse.
If the target is a HyperSource file, move to the exact target position.
Otherwise, just find the target file."
   (interactive "e")
   (mouse-set-point event)
   (hypersrc-follow-explicit-link))


(defun hypersrc-delete-explicit-link ()
  "Delete the explicit link under point.
If the target is a HyperSource file, delete the target extent as well."
  (interactive)
  (let (link target-file-name id-to-find)					; Local variables
    (setq link (extent-at (point) nil 'target-file))
    (setq target-file-name (expand-file-name (extent-property link 'target-file)))
    (setq id-to-find (extent-property link 'target-id))
    (if (y-or-n-p "Delete link? ")
	(progn
	  (delete-extent link)
	  (if id-to-find
	      (save-excursion
		(set-buffer (find-file-noselect target-file-name))
		(delete-extent
		 (map-extents
		  (function
		   (lambda
		     (extent maparg)
		     (if
			 (and
			  (extent-property extent 'hypersrc)   ; extent is from hypersrc
			  (extent-property extent 'target-id)  ; extent is a link target
			  (string= (extent-property extent 'target-id) id-to-find)) ; right ID
			 extent)))))))))))


(defun hypersrc-delete-explicit-link-under-mouse (event)
  "Delete the explicit link under the mouse.
If the target is a HyperSource file, delete the target extent as well."
   (interactive "e")
   (mouse-set-point event)
   (hypersrc-delete-explicit-link))

			   
(defun hypersrc-follow-implicit-link (event &optional other-window)
  "Find target of an implicit link in HyperSource mode.
Uses 'find-tag' to locate target of symbol under mouse, but looks for
first exact match only.
Optional argument means open target file in other window."
  ;; find-tag does exact match if tag is put into a list!
  (interactive "e")
  (let (symbol-start tag-list)							; Local variables
    (save-excursion
      (mouse-set-point event)
      (skip-syntax-backward "w_")
      (setq symbol-start (point))
      (skip-syntax-forward "w_")
      (setq symbol-end (point)))
    (setq tag-list (list (buffer-substring symbol-start symbol-end)))
    (message "Following link %s" (car tag-list))
    (if other-window
	(find-tag-other-window tag-list)
      (find-tag tag-list))))


(defun hypersrc-set-margin-width (width)
  "Set width (in characters) of left margin for current buffer.
To change the default width for all buffers to be, say, 30 chars, add the following to
your .emacs file:
  (add-hook 'hypersrc-mode-hook
            (function (lambda () (setq-default hypersrc-left-margin-width 30))))"
  (interactive "nNew margin width (in characters): ")
  (setq hypersrc-left-margin-width width)
  (set-buffer-left-margin-width width))


(provide 'hypersrc)

;;; hypersrc.el ends here
